use Test::More tests => 2;

BEGIN {
use_ok( 'Religion::Bible::Regex::Config' );
use_ok( 'Religion::Bible::Regex::Builder' );
}

diag( "Testing Religion::Bible::Regex::Builder $Religion::Bible::Regex::Builder::VERSION" );
